( cd usr/lib ; rm -rf libgretl-1.0.so )
( cd usr/lib ; ln -sf libgretl-1.0.so.0.0.50 libgretl-1.0.so )
( cd usr/lib ; rm -rf libgretl-1.0.so.0 )
( cd usr/lib ; ln -sf libgretl-1.0.so.0.0.50 libgretl-1.0.so.0 )
